@@ -6,7 +6,7 @@
  *
  * Copyright (c) 2004, Hannu Saransaari and Lauri Hakkarainen
  * Copyright (c) 2007 Toni Spets
- * Copyright (c) 2005-2021 Brenden Matthews, Philip Kovacs, et. al.
+ * Copyright (c) 2005-2024 Brenden Matthews, Philip Kovacs, et. al.
  *	(see AUTHORS)
  * All rights reserved.
  *
@@ -192,7 +192,7 @@ int update_meminfo(void) {
   info.memmax = info.memdirty = info.swap = info.swapfree = info.swapmax =
       info.memwithbuffers = info.buffers = info.cached = info.memfree =
           info.memeasyfree = info.legacymem = info.shmem = info.memavail =
-              info.free_bufcache = 0;
+              info.free_bufcache = info.free_cached = 0;
 
   if (!(meminfo_fp = open_file("/proc/meminfo", &reported))) { return 0; }
 
@@ -261,7 +261,8 @@ int update_meminfo(void) {
   info.memeasyfree = cureasyfree;
   info.legacymem =
       info.memmax - (info.memfree + info.buffers + info.cached + sreclaimable);
-  info.free_bufcache = info.cached + info.buffers + sreclaimable;
+  info.free_cached = info.cached + sreclaimable;
+  info.free_bufcache = info.free_cached + info.buffers;
 
   fclose(meminfo_fp);
   return 0;
@@ -1843,7 +1844,7 @@ static double last_acpi_temp_time;
 
 // the maximum length of the string inside a ACPI_THERMAL_FORMAT file including
 // the ending 0
-#define MAXTHERMZONELEN 6
+#define MAXTHERMZONELEN 7
 
 double get_acpi_temperature(int fd) {
   if (fd <= 0) { return 0; }
